import $ from 'jquery'
import { Plugin } from 'foundation-sites/js/foundation.core.plugin'

/**
 * FileUpload module.
 *
 * @module foundation.fileUpload
 */

class FileUpload extends Plugin {
  /**
   * Creates a new instance.
   * @class
   * @name FileUpload
   * @fires FileUpload#init
   * @param {JQuery} element - jQuery object representing FileUpload wrapper.
   * @param {object} options - Overrides to the default plugin settings.
   */
  _setup(element, options = {}) {
    this.$element = element
    this.className = 'FileUpload'
    // defaults, fallbacks
    this.options = $.extend(true, {}, FileUpload.defaults, this.$element.data(), options)
    // find related elements
    this.$input = element.find('input[type="file"]')
    this.$upload = this._find('upload') || $(`label[for="#${this.$input.attr('id')}"]`)
    this.$reset = this._find('reset')
    // if there is a help text outside, move it inside
    element.nextAll('.help-text').detach().appendTo(element)

    this._log('setup', this)
    this._init()
  }

  /**
   * Finds an element passed as option or marked with a matching data attribute.
   * @function
   * @param {string} option - The name of the option to look for.
   * @private
   */
  _find(option) {
    let $element = $()
    if (this.options[option]) {
      $element = $(`#${this.options[option]}`)
    }
    if (!$element.length) {
      $element = this.$element.find(`[data-${option}]`)
    }
    if (!$element.length) {
      // Found nothing.
      return null
    }
    this._log(`found ${option} element`, $element)
    return $element
  }

  /**
   * Registers event handlers and views.
   * @function
   * @private
   */
  _init() {
    if (this.$reset) {
      this.$reset.hide().on('click', this.reset.bind(this))
    }
    this.$input.on('change.zf.fileUpload', this._displayFile.bind(this))
    this.$element.on('drop.zf.fileUpload', (e) => this._dropFiles(e))
    this.$element.on('dragover.zf.fileUpload', (e) => e.preventDefault())
    this.$element.on('focusin.zf.fileUpload', () => this.$element.addClass('focus'))
    this.$element.on('focusout.zf.fileUpload', () => this.$element.removeClass('focus'))
    this._displayFile()
    this._log('init', this)
  }

  /**
   * Drop event handler: drag’n’drop files instead of using the file input.
   * @function
   * @param {Event} e - The event object.
   * @private
   */
  _dropFiles (e) {
    e.preventDefault(e)
    const files = e.originalEvent.dataTransfer.files
    if (files.length) {
      this._log('dropped', files)
      this.$input.prop('files', files)

      // Dispatch a native change event (jQuery only works within the same instance of jQuery).
      const change = new Event('change')
      this.$input[0].dispatchEvent(change)
    }
  }

  /**
   * Displays the chosen file name.
   * @function
   * @private
   * @fires fileUpload#selected
   */
  _displayFile () {
    const files = this.$input.prop('files')
    if (!(files && files.length)) {
      this._log('no file selected', this)
      return
    }
    this.$element.find('.text-input').remove()

    /**
     * Fires when the file upload has been reset.
     * @event fileUpload#selected
     */
    this.$element.trigger('selected.zf.fileUpload', [files])

    this.$element.addClass('file-selected')
    for (const f of files) {
      this._log(`display ${f.name}`, files, this)
      this.$input.after(`<div class="text-input">${f.name}<div>`)
    }
    if (this.$reset) {
      this.$upload.hide()
      this.$reset.show()
    }
  }

  /**
   * Resets the file upload field to it's original (empty) state.
   * @function
   * @fires fileUpload#reset
   */
  reset () {
    const prevValue = this.$input.prop('files')
    this._log('reset', prevValue, this)

    this.$element.removeClass('file-selected')
    this.$input.val('')
    this.$element.find('.text-input').remove()

    if (this.$reset) {
      this.$reset.hide()
      this.$upload.show()
    }

    /**
     * Fires when the file upload has been reset.
     * @event fileUpload#reset
     */
    this.$element.trigger('reset.zf.fileUpload', [prevValue])
  }

  /**
   * Logs debug output to the console.
   * @function
   * @private
   */
  _log () {
    if (this.options.debug) {
      console.debug('[FileUpload]', ...arguments)
    }
  }

  /**
   * Destroys an instance.
   * @function
   */
  _destroy() {
    $(window).off('.zf.fileUpload')
    this.$uplad.add(this.$file).add(this.$reset).css('display', '')
    this._log('destroyed', this)
  }
}

/**
 * Default settings for plugin.
 */
FileUpload.defaults = {
  /**
   * The label element that acts as upload trigger.
   * @option
   * @type {JQuery|null}
   * @default null
   */
  upload: null,

  /**
   * The element that resets the file upload on click.
   * @option
   * @type {JQuery|null}
   * @default null
   */
  reset: null,

  /**
   * Toggle debug output.
   * @option
   * @type {boolean}
   * @default false
   */
  debug: false
}

export { FileUpload }
