import { storageAvailable } from '@moreonion/js-utils/src/utils'

/**
 * Provides helper classes to simplify working with localStorage and SessionStorage.
 *
 * @module storeWrapper
 */

class Store {
  /**
   * Wrapper class for any storage.
   * @class
   * @name Store
   * @param {string} storage - Which storage to use (localStorage or sessionStorage).
   * @param {string} name - Identifier for the store.
   */
  constructor(storage, name) {
    this.name = name
    this.store = storageAvailable(storage) ? window[storage] : {}
    // Make sure the store exists
    if (this.store[name] === undefined) {
      this.write(null)
    }
  }

  /**
   * Writes a new value to the store.
   * @function
   * @param {string|object} value - The new value.
   */
  write(value) {
    this.store[this.name] = JSON.stringify(value)
  }

  /**
   * Reads the current value from the store.
   * @function
   * @return {string|object} - The current value.
   */
  read() {
    return JSON.parse(this.store[this.name])
  }
}

class LocalStore extends Store {
  /**
   * Wrapper class for accessing localStorage.
   * @class
   * @name LocalStore
   * @param {string} name - Identifier for the data in localStorage.
   */
  constructor(name) {
    super('localStorage', name)
  }
}

class SessionStore extends Store {
  /**
   * Wrapper class for accessing sessionStorage.
   * @class
   * @name SessionStore
   * @param {string} name - Identifier for the data in sessionStorage.
   */
  constructor(name) {
    super('sessionStorage', name)
  }
}

export { LocalStore, SessionStore }
